# Generated by Django 3.2 on 2021-12-29 18:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_bootstrap3', '0014_translations_update'),
    ]

    operations = [
        migrations.AlterField(
            model_name='boostrap3blockquoteplugin',
            name='reverse',
            field=models.BooleanField(blank=True, default=False, help_text='Reverses the position by adding the Bootstrap 3 "blockquote-reverse" class.', verbose_name='Reverse quote'),
        ),
        migrations.AlterField(
            model_name='boostrap3imageplugin',
            name='img_responsive',
            field=models.BooleanField(blank=True, default=True, help_text='Adds the Bootstrap 3 ".img-responsive" class.', verbose_name='.img-responsive'),
        ),
        migrations.AlterField(
            model_name='boostrap3imageplugin',
            name='thumbnail',
            field=models.BooleanField(blank=True, default=False, help_text='Adds the Bootstrap 3 ".img-thumbnail" class.', verbose_name='.img-thumbnail'),
        ),
        migrations.AlterField(
            model_name='boostrap3imageplugin',
            name='use_original_image',
            field=models.BooleanField(blank=True, default=False, help_text='Outputs the raw image without cropping.', verbose_name='Use original image'),
        ),
        migrations.AlterField(
            model_name='boostrap3jumbotronplugin',
            name='grid',
            field=models.BooleanField(blank=True, default=False, help_text='Adds a ".container" element inside the "Jumbotron" for use outside of a grid.', verbose_name='Add container'),
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselplugin',
            name='pause',
            field=models.BooleanField(blank=True, default=True, help_text='Pauses the carousel on hover.', verbose_name='Pause'),
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselplugin',
            name='wrap',
            field=models.BooleanField(blank=True, default=True, help_text='Loops carousel animation.', verbose_name='Wrap'),
        ),
        migrations.AlterField(
            model_name='bootstrap3listgroupplugin',
            name='add_list_group_class',
            field=models.BooleanField(blank=True, default=True, help_text='Adds the list-group and subsequent list-group-item classes.', verbose_name='.list-group'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models
import filer.fields.image
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_bootstrap3', '0003_auto_20151113_1604'),
    ]

    operations = [
        migrations.AlterField(
            model_name='boostrap3buttonplugin',
            name='link_mailto',
            field=models.EmailField(max_length=254, null=True, verbose_name='mailto', blank=True),
        ),
        migrations.AlterField(
            model_name='boostrap3imageplugin',
            name='file',
            field=filer.fields.image.FilerImageField(related_name='+', on_delete=django.db.models.deletion.SET_NULL, verbose_name='file', to='filer.Image', null=True),
        ),
        migrations.AlterField(
            model_name='boostrap3imageplugin',
            name='override_height',
            field=models.IntegerField(help_text='if this field is provided it will be used to scale image. If aspect ration is selected - height will be calculated based on that.', null=True, verbose_name='override height', blank=True),
        ),
        migrations.AlterField(
            model_name='boostrap3imageplugin',
            name='override_width',
            field=models.IntegerField(help_text='if this field is provided it will be used to scale image.', null=True, verbose_name='override width', blank=True),
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselslideplugin',
            name='image',
            field=filer.fields.image.FilerImageField(related_name='+', on_delete=django.db.models.deletion.SET_NULL, verbose_name='image', to='filer.Image', null=True),
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselslideplugin',
            name='link_mailto',
            field=models.EmailField(max_length=254, null=True, verbose_name='mailto', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-


from django import template
from django.template.defaultfilters import stringfilter

register = template.Library()


@register.filter(name="iconset_from_class")
@stringfilter
def iconset_from_class(value):
    """
    extracts the iconset from a class definition
    "fa-flask" -> "fa"
    :param value:
    :return:
    """
    if "-" in value:
        return value.split("-")[0]
    return ""



# -*- coding: utf-8 -*-


from functools import partial

import cms.models
import cms.models.fields
import django.db.models
import django.forms
import filer.fields.file
import filer.fields.image
from cms.models.pluginmodel import CMSPlugin
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import force_text
from django.utils.translation import ugettext, ugettext_lazy as _
from djangocms_attributes_field.fields import AttributesField

from . import fields, constants
from .conf import settings


def get_additional_styles():
    """
    Get additional styles choices from settings
    """
    choices = []
    raw = getattr(settings, "ALDRYN_BOOTSTRAP3_CAROUSEL_STYLES", getattr(settings, "GALLERY_STYLES", False))
    if raw:
        if isinstance(raw, str):
            raw = raw.split(",")
            for choice in raw:
                clean = choice.strip()
                choices.append((clean.lower(), clean.title()))
        else:
            for choice in raw:
                choices.append(choice)
    return choices


# Add an app namespace to related_name to avoid field name clashes
# with any other plugins that have a field with the same name as the
# lowercase of the class name of this model.
# https://github.com/divio/django-cms/issues/5030
CMSPluginField = partial(
    models.OneToOneField,
    to=CMSPlugin,
    related_name="%(app_label)s_%(class)s",
    parent_link=True,
    on_delete=models.CASCADE,
)


# Helper for:
# Classes, LinkOrButton, Size, IntegerField
class SouthMixinBase(object):
    south_field_class = ""

    def south_field_triple(self):
        """Returns a suitable description of this field for South."""
        if not self.south_field_class:
            raise NotImplementedError("Please set south_field_class when " "using the south field mixin.")
        # We'll just introspect ourselves, since we inherit.
        from south.modelsinspector import introspector

        field_class = self.south_field_class
        args, kwargs = introspector(self)
        # That's our definition!
        return field_class, args, kwargs


# Code here is mostly used in `models.py` and `migrations/`


class Classes(django.db.models.TextField, SouthMixinBase):
    default_field_class = fields.Classes
    south_field_class = "django.db.models.fields.TextField"

    def __init__(self, *args, **kwargs):
        if "verbose_name" not in kwargs:
            kwargs["verbose_name"] = _("Classes")
        if "blank" not in kwargs:
            kwargs["blank"] = True
        if "default" not in kwargs:
            kwargs["default"] = ""
        if "help_text" not in kwargs:
            kwargs["help_text"] = _(
                "Space separated classes that are added to "
                'the class. See <a href="http://getbootstrap.com/css/" '
                'target="_blank">Bootstrap 3 documentation</a>.'
            )
        super(Classes, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Classes, self).formfield(**defaults)


class LinkMixin(models.Model):
    link_url = models.URLField(
        verbose_name=_("External link"),
        blank=True,
        default="",
        help_text=_("Provide a valid URL to an external website."),
    )
    link_page = cms.models.fields.PageField(
        verbose_name=_("Internal link"),
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        help_text=_("If provided, overrides the external link."),
    )
    link_mailto = models.EmailField(verbose_name=_("Email address"), blank=True, null=True, max_length=255,)
    link_phone = models.CharField(verbose_name=_("Phone"), blank=True, null=True, max_length=255,)
    link_anchor = models.CharField(
        verbose_name=_("Anchor"),
        max_length=255,
        blank=True,
        help_text=_(
            "Appends the value only after the internal or external link. "
            'Do <em>not</em> include a preceding "#" symbol.'
        ),
    )
    link_target = models.CharField(
        verbose_name=_("Target"), choices=constants.TARGET_CHOICES, blank=True, max_length=255,
    )
    link_file = filer.fields.file.FilerFileField(
        verbose_name=_("File"), null=True, blank=True, on_delete=models.SET_NULL,
    )
    link_attributes = AttributesField(
        verbose_name=_("Attributes"), blank=True, excluded_keys=["class", "href", "target"],
    )

    class Meta:
        abstract = True

    def get_link_url(self):
        if self.link_page:
            ref_page = self.link_page
            link = ref_page.get_absolute_url()

            if not getattr(ref_page, "site_id", None) is None and getattr(ref_page, "site_id", None) != getattr(
                self.page, "site_id", None
            ):
                ref_site = Site.objects._get_site_by_id(ref_page.site_id)
                link = "//{}{}".format(ref_site.domain, link)
        elif self.link_url:
            link = self.link_url
        elif self.link_phone:
            link = "tel:{}".format(self.link_phone.replace(" ", ""))
        elif self.link_mailto:
            link = "mailto:{}".format(self.link_mailto)
        elif self.link_file:
            link = self.link_file.url
        else:
            link = ""
        if self.link_anchor:
            link += "#{}".format(self.link_anchor)
        return link

    def clean(self):
        super(LinkMixin, self).clean()
        field_names = (
            "link_url",
            "link_page",
            "link_mailto",
            "link_phone",
            "link_file",
        )
        anchor_field_name = "link_anchor"
        field_names_allowed_with_anchor = (
            "link_url",
            "link_page",
            "link_file",
        )

        anchor_field_verbose_name = force_text(self._meta.get_field(anchor_field_name).verbose_name)
        anchor_field_value = getattr(self, anchor_field_name)

        link_fields = {key: getattr(self, key) for key in field_names}
        link_field_verbose_names = {
            key: force_text(self._meta.get_field(key).verbose_name) for key in list(link_fields.keys())
        }
        provided_link_fields = {key: value for key, value in list(link_fields.items()) if value}

        required_link_classes = ("Boostrap3ButtonPlugin",)

        if len(provided_link_fields) > 1:
            # Too many fields have a value.
            verbose_names = sorted(link_field_verbose_names.values())
            error_msg = _("Only one of {0} or {1} may be given.").format(
                ", ".join(verbose_names[:-1]), verbose_names[-1],
            )
            errors = {}.fromkeys(list(provided_link_fields.keys()), error_msg)
            raise ValidationError(errors)

        if self.__class__.__name__ in required_link_classes:
            if len(provided_link_fields) == 0 and not self.link_anchor:
                raise ValidationError(_("Please provide a link."))

        if anchor_field_value:
            for field_name in list(provided_link_fields.keys()):
                if field_name not in field_names_allowed_with_anchor:
                    error_msg = _("%(anchor_field_verbose_name)s is not allowed together with %(field_name)s.") % {
                        "anchor_field_verbose_name": anchor_field_verbose_name,
                        "field_name": link_field_verbose_names.get(field_name),
                    }
                    raise ValidationError(
                        {anchor_field_name: error_msg, field_name: error_msg,}
                    )


class LinkOrButton(django.db.models.fields.CharField, SouthMixinBase):
    default_field_class = fields.LinkOrButton
    south_field_class = "django.db.models.fields.CharField"

    def __init__(self, *args, **kwargs):
        if "verbose_name" not in kwargs:
            kwargs["verbose_name"] = ugettext("Type")
        if "max_length" not in kwargs:
            kwargs["max_length"] = 255
        if "blank" not in kwargs:
            kwargs["blank"] = False
        if "default" not in kwargs:
            kwargs["default"] = self.default_field_class.DEFAULT
        super(LinkOrButton, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
            "choices_form_class": self.default_field_class,
        }
        defaults.update(kwargs)
        return super(LinkOrButton, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if "" in list(dict(self.choices).keys()):
            kwargs["include_blank"] = False
        return super(LinkOrButton, self).get_choices(**kwargs)


class Context(django.db.models.fields.CharField):
    default_field_class = fields.Context
    south_field_class = "django.db.models.fields.CharField"

    def __init__(self, *args, **kwargs):
        if "verbose_name" not in kwargs:
            kwargs["verbose_name"] = ugettext("Context")
        if "max_length" not in kwargs:
            kwargs["max_length"] = 255
        if "blank" not in kwargs:
            kwargs["blank"] = False
        if "default" not in kwargs:
            kwargs["default"] = self.default_field_class.DEFAULT
        super(Context, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
            "choices_form_class": self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Context, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if "" in list(dict(self.choices).keys()):
            kwargs["include_blank"] = False
        return super(Context, self).get_choices(**kwargs)


class Icon(django.db.models.CharField):
    default_field_class = fields.Icon
    south_field_class = "django.db.models.fields.CharField"

    def __init__(self, *args, **kwargs):
        if "verbose_name" not in kwargs:
            kwargs["verbose_name"] = ugettext("Icon")
        if "max_length" not in kwargs:
            kwargs["max_length"] = 255
        if "blank" not in kwargs:
            kwargs["blank"] = True
        if "default" not in kwargs:
            kwargs["default"] = self.default_field_class.DEFAULT
        super(Icon, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Icon, self).formfield(**defaults)


class MiniText(django.db.models.TextField):
    default_field_class = fields.MiniText
    south_field_class = "django.db.models.fields.TextField"

    def __init__(self, *args, **kwargs):
        if "blank" not in kwargs:
            kwargs["blank"] = True
        if "default" not in kwargs:
            kwargs["default"] = ""
        super(MiniText, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
        }
        defaults.update(kwargs)
        return super(MiniText, self).formfield(**defaults)


class Responsive(MiniText):
    default_field_class = fields.Responsive

    def __init__(self, *args, **kwargs):
        if "verbose_name" not in kwargs:
            kwargs["verbose_name"] = ugettext("Responsive")
        if "blank" not in kwargs:
            kwargs["blank"] = True
        if "default" not in kwargs:
            kwargs["default"] = ""
        super(Responsive, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Responsive, self).formfield(**defaults)


class Size(django.db.models.CharField, SouthMixinBase):
    default_field_class = fields.Size
    south_field_class = "django.db.models.fields.CharField"

    def __init__(self, *args, **kwargs):
        if "verbose_name" not in kwargs:
            kwargs["verbose_name"] = ugettext("Context")
        if "max_length" not in kwargs:
            kwargs["max_length"] = 255
        if "blank" not in kwargs:
            kwargs["blank"] = True
        if "default" not in kwargs:
            kwargs["default"] = self.default_field_class.DEFAULT
        super(Size, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
            "choices_form_class": self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Size, self).formfield(**defaults)

    def get_choices(self, **kwargs):
        # if there already is a "blank" choice, don't add another
        # default blank choice
        if "" in list(dict(self.choices).keys()):
            kwargs["include_blank"] = False
        return super(Size, self).get_choices(**kwargs)


class IntegerField(django.db.models.IntegerField, SouthMixinBase):
    default_field_class = fields.Integer
    south_field_class = "django.db.models.fields.IntegerField"

    def __init__(self, verbose_name=None, name=None, min_value=None, max_value=None, **kwargs):
        self.min_value, self.max_value = min_value, max_value
        django.db.models.IntegerField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
            "min_value": self.min_value,
            "max_value": self.max_value,
        }
        defaults.update(kwargs)
        return super(IntegerField, self).formfield(**defaults)


class ResponsivePrint(MiniText):
    default_field_class = fields.ResponsivePrint

    def __init__(self, *args, **kwargs):
        if "blank" not in kwargs:
            kwargs["blank"] = True
        if "default" not in kwargs:
            kwargs["default"] = ""
        super(ResponsivePrint, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_field_class,
        }
        defaults.update(kwargs)
        return super(ResponsivePrint, self).formfield(**defaults)


# TODO:
#   * btn-block, disabled
#   * pull-left, pull-right
#   * margins/padding



# -*- coding: utf-8 -*-


import json
import warnings

from django.conf.urls import url
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse
from django.templatetags.static import static
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt

from cms.models import CMSPlugin
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

try:
    from filer.admin.clipboardadmin import ajax_upload as filer_ajax_upload
except ImportError:
    filer_ajax_upload = None
    warnings.warn("Drag and drop functionality is not avalable. " "Please update to django-filer>=1.1.1", Warning)

from . import models, forms, constants


class Bootstrap3RowCMSPlugin(CMSPluginBase):
    """
    CSS - Grid system: "Row" Plugin
    http://getbootstrap.com/css/#grid
    """

    model = models.Bootstrap3RowPlugin
    name = _("Row")
    module = _("Bootstrap 3")
    form = forms.RowPluginForm
    change_form_template = "admin/aldryn_bootstrap3/plugins/row/change_form.html"
    render_template = "aldryn_bootstrap3/plugins/row.html"
    allow_children = True
    child_classes = ["Bootstrap3ColumnCMSPlugin"]

    fieldsets = [
        (
            _("Create columns"),
            {
                "fields": ("create",)
                + tuple(
                    [
                        (
                            "create_{}_col".format(size),
                            "create_{}_offset".format(size),
                            "create_{}_push".format(size),
                            "create_{}_pull".format(size),
                        )
                        for size in constants.DEVICE_SIZES
                    ]
                )
            },
        ),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",)}),
    ]

    def save_model(self, request, obj, form, change):
        response = super(Bootstrap3RowCMSPlugin, self).save_model(request, obj, form, change)
        data = form.cleaned_data
        for x in range(int(data["create"]) if data["create"] is not None else 0):
            extra = {}
            for size in constants.DEVICE_SIZES:
                for element in ["col", "offset", "push", "pull"]:
                    extra["{}_{}".format(size, element)] = data.get("create_{}_{}".format(size, element))
            col = models.Bootstrap3ColumnPlugin(
                parent=obj,
                placeholder=obj.placeholder,
                language=obj.language,
                position=CMSPlugin.objects.filter(parent=obj).count(),
                plugin_type=Bootstrap3ColumnCMSPlugin.__name__,
                **extra
            )
            col.save()
        return response


class Bootstrap3ColumnCMSPlugin(CMSPluginBase):
    """
    CSS - Grid system: "Column" Plugin
    http://getbootstrap.com/css/#grid
    """

    model = models.Bootstrap3ColumnPlugin
    name = _("Column")
    module = _("Bootstrap 3")
    form = forms.ColumnPluginForm
    change_form_template = "admin/aldryn_bootstrap3/plugins/column/change_form.html"
    render_template = "aldryn_bootstrap3/plugins/column.html"
    allow_children = True
    parent_classes = ["Bootstrap3RowCMSPlugin"]

    fieldsets = [
        (
            _("Adjust columns"),
            {
                "fields": tuple(
                    [
                        (
                            "{}_col".format(size),
                            "{}_offset".format(size),
                            "{}_push".format(size),
                            "{}_pull".format(size),
                        )
                        for size in constants.DEVICE_SIZES
                    ]
                )
            },
        ),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes", "tag",)}),
    ]


class Bootstrap3BlockquoteCMSPlugin(CMSPluginBase):
    """
    CSS - Typography: "Blockquote" Plugin
    http://getbootstrap.com/css/#type-blockquotes
    """

    model = models.Boostrap3BlockquotePlugin
    name = _("Blockquote")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/blockquote.html"
    allow_children = True

    fieldsets = [
        (None, {"fields": ("reverse",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",)}),
    ]


class Bootstrap3CiteCMSPlugin(CMSPluginBase):
    """
    CSS - Typography: "Cite" Plugin
    http://getbootstrap.com/css/#type-blockquotes
    """

    model = models.Boostrap3CitePlugin
    name = _("Cite")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/cite.html"
    allow_children = True
    # even though we can make this accessible by all means, we only want
    # to restrict it to the blockquote plugin for now
    require_parent = True
    parent_classes = ["Bootstrap3BlockquoteCMSPlugin"]

    fieldsets = [
        (None, {"fields": ()}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",)}),
    ]


class Bootstrap3CodeCMSPlugin(CMSPluginBase):
    """
    CSS - Code: Model
    http://getbootstrap.com/css/#code
    """

    model = models.Bootstrap3CodePlugin
    name = _("Code")
    module = _("Bootstrap 3")
    form = forms.Bootstrap3CodePluginForm
    change_form_template = "admin/aldryn_bootstrap3/plugins/code/change_form.html"
    render_template = "aldryn_bootstrap3/plugins/code.html"
    text_enabled = True

    fieldsets = (
        (None, {"fields": ("code_type", "code",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",)}),
    )


class Bootstrap3ButtonCMSPlugin(CMSPluginBase):
    """
    CSS - Buttons: "Button/Link" Plugin
    http://getbootstrap.com/css/#buttons
    """

    model = models.Boostrap3ButtonPlugin
    name = _("Link/Button")
    module = _("Bootstrap 3")
    form = forms.LinkForm
    change_form_template = "admin/aldryn_bootstrap3/plugins/button/change_form.html"
    render_template = "aldryn_bootstrap3/plugins/button.html"
    text_enabled = True
    allow_children = True

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "type",
                    "label",
                    ("link_url", "link_page",),
                    "btn_context",
                    "txt_context",
                    "btn_size",
                    ("icon_left", "icon_right", "btn_block",),
                ),
            },
        ),
        (
            _("Link settings"),
            {
                "classes": ("collapse",),
                "fields": (("link_mailto", "link_phone"), ("link_anchor", "link_target"), "link_file",),
            },
        ),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "link_attributes",)}),
    )

    @classmethod
    def get_render_queryset(cls):
        queryset = super(Bootstrap3ButtonCMSPlugin, cls).get_render_queryset()
        return queryset.select_related("link_page")

    def icon_src(self, instance):
        return static("aldryn_bootstrap3/img/type/button.png")


class Bootstrap3ImageCMSPlugin(CMSPluginBase):
    """
    CSS - Images: Plugin
    http://getbootstrap.com/css/#images
    """

    model = models.Boostrap3ImagePlugin
    name = _("Image")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/image.html"
    text_enabled = True

    fieldsets = (
        (None, {"fields": ("file", "alt", ("use_original_image", "thumbnail",), ("aspect_ratio", "shape",),)}),
        (
            _("Advanced settings"),
            {
                "classes": ("collapse",),
                "fields": ("title", ("override_width", "override_height",), "img_responsive", "classes", "attributes",),
            },
        ),
    )

    def render(self, context, instance, placeholder):
        context.update({"instance": instance})
        if callable(filer_ajax_upload):
            # Use this in template to conditionally enable drag-n-drop.
            context.update({"has_dnd_support": True})
        return context

    def get_thumbnail(self, instance):
        return instance.file.file.get_thumbnail(
            {"size": (40, 40), "crop": True, "upscale": True, "subject_location": instance.file.subject_location,}
        )

    def icon_src(self, instance):
        if instance.file_id:
            thumbnail = self.get_thumbnail(instance)
            return thumbnail.url
        return ""

    def get_plugin_urls(self):
        urlpatterns = [
            url(r"^ajax_upload/(?P<pk>[0-9]+)/$", self.ajax_upload, name="bootstrap3_image_ajax_upload"),
        ]
        return urlpatterns

    @csrf_exempt
    def ajax_upload(self, request, pk):
        """
        Handle drag-n-drop uploads.

        Call original 'ajax_upload' Filer view, parse response and update
        plugin instance file_id from it. Send original response back.
        """
        if not callable(filer_ajax_upload):
            # Do not try to handle request if we were unable to
            # import Filer view.
            raise ImproperlyConfigured("Please use django-filer>=1.1.1 to get drag & drop support.")
        filer_response = filer_ajax_upload(request, folder_id=None)

        if filer_response.status_code != 200:
            return filer_response

        try:
            file_id = json.loads(filer_response.content)["file_id"]
        except ValueError:
            return HttpResponse(
                json.dumps({"error": "Received non-JSON response from django Filer."}),
                status=500,
                content_type="application/json",
            )
        instance = self.model.objects.get(pk=pk)
        instance.file_id = file_id
        instance.save()
        return filer_response


class Bootstrap3ResponsiveCMSPlugin(CMSPluginBase):
    """
    CSS - Responsive: "Utilities" Plugin
    http://getbootstrap.com/css/#responsive-utilities
    """

    model = models.Bootstrap3ResponsivePlugin
    name = _("Responsive utilities")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/responsive.html"
    allow_children = True

    fieldsets = (
        (None, {"fields": ("device_breakpoints", "print_breakpoints",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3IconCMSPlugin(CMSPluginBase):
    """
    Component - Glyphicons: "Icon" Plugin
    http://getbootstrap.com/components/#glyphicons
    Component - Font Awesome: "Icon" Plugin
    http://fontawesome.io/
    """

    model = models.Boostrap3IconPlugin
    name = _("Icon")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/icon.html"
    text_enabled = True

    fieldsets = (
        (None, {"fields": ("icon",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def icon_src(self, instance):
        return static("aldryn_bootstrap3/img/type/icon.png")


class Bootstrap3LabelCMSPlugin(CMSPluginBase):
    """
    Component - Label: Plugin
    http://getbootstrap.com/components/#labels
    """

    model = models.Boostrap3LabelPlugin
    name = _("Label")
    module = _("Bootstrap 3")
    form = forms.Boostrap3LabelPluginForm
    change_form_template = "admin/aldryn_bootstrap3/plugins/label/change_form.html"
    render_template = "aldryn_bootstrap3/plugins/label.html"
    text_enabled = True

    fieldsets = (
        (None, {"fields": ("label", "context",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def icon_src(self, instance):
        return static("aldryn_bootstrap3/img/type/label.png")


class Bootstrap3JumbotronCMSPlugin(CMSPluginBase):
    """
    Component - Jumbotron: Plugin
    http://getbootstrap.com/components/#jumbotron
    """

    model = models.Boostrap3JumbotronPlugin
    name = _("Jumbotron")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/jumbotron.html"
    allow_children = True

    fieldsets = (
        (None, {"fields": ("label", "grid",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3AlertCMSPlugin(CMSPluginBase):
    """
    Component - Alert: Plugin
    http://getbootstrap.com/components/#alerts
    """

    model = models.Boostrap3AlertPlugin
    name = _("Alert")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/alert.html"
    allow_children = True

    fieldsets = (
        (None, {"fields": ("context", "icon",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3ListGroupCMSPlugin(CMSPluginBase):
    """
    Component - List group: "Wrapper" Plugin
    http://getbootstrap.com/components/#alerts
    """

    model = models.Bootstrap3ListGroupPlugin
    name = _("List Group")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/list_group.html"
    allow_children = True
    child_classes = ["Bootstrap3ListGroupItemCMSPlugin"]

    fieldsets = (
        (None, {"fields": ("add_list_group_class",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3ListGroupItemCMSPlugin(CMSPluginBase):
    """
    Component - List group: "Item" Plugin
    http://getbootstrap.com/components/#alerts
    """

    model = models.Bootstrap3ListGroupItemPlugin
    name = _("List Group Item")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/list_group_item.html"
    allow_children = True
    parent_classes = [
        "Bootstrap3ListGroupCMSPlugin",
        "Bootstrap3PanelCMSPlugin",
    ]

    fieldsets = (
        (None, {"fields": ("title", "context", "state",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3ListGroupItemCMSPlugin, self).render(context, instance, placeholder)
        context["item"] = instance
        return context


class Bootstrap3PanelCMSPlugin(CMSPluginBase):
    """
    Component - Panel: "Wrapper" Plugin
    http://getbootstrap.com/components/#panels
    """

    model = models.Boostrap3PanelPlugin
    name = _("Panel")
    module = _("Bootstrap 3")
    form = forms.PanelPluginBaseForm
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/panel.html"
    allow_children = True
    child_classes = [
        "Bootstrap3PanelHeadingCMSPlugin",
        "Bootstrap3PanelBodyCMSPlugin",
        "Bootstrap3PanelFooterCMSPlugin",
    ]

    fieldsets = (
        (_("Create"), {"fields": ("create_heading", "create_body", "create_footer",)}),
        (_("Settings"), {"fields": ("context",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def save_model(self, request, obj, form, change):
        response = super(Bootstrap3PanelCMSPlugin, self).save_model(request, obj, form, change)
        data = form.cleaned_data
        extra = {}
        subplugins = (
            ("create_heading", models.Boostrap3PanelHeadingPlugin, Bootstrap3PanelHeadingCMSPlugin),
            ("create_body", models.Boostrap3PanelBodyPlugin, Bootstrap3PanelBodyCMSPlugin),
            ("create_footer", models.Boostrap3PanelFooterPlugin, Bootstrap3PanelFooterCMSPlugin),
        )
        existing_plugins = [p.plugin_type for p in obj.get_children()]
        for field, model_class, plugin_class in subplugins:
            if not data.get(field):
                # TODO: delete?
                continue
            if plugin_class.__name__ in existing_plugins:
                continue
            plugin = model_class(
                parent=obj,
                placeholder=obj.placeholder,
                language=obj.language,
                position=CMSPlugin.objects.filter(parent=obj).count(),
                plugin_type=plugin_class.__name__,
                **extra
            )
            plugin.save()
        return response


class Bootstrap3PanelHeadingCMSPlugin(CMSPluginBase):
    """
    Component - Panel: "Heading" Plugin
    http://getbootstrap.com/components/#panels-heading
    """

    model = models.Boostrap3PanelHeadingPlugin
    name = _("Panel header")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/panel_heading.html"
    allow_children = True
    require_parent = True
    parent_classes = ["Bootstrap3PanelCMSPlugin"]

    fieldsets = (
        (None, {"fields": ("title",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3PanelBodyCMSPlugin(CMSPluginBase):
    """
    Component - Panel: "Body" Plugin
    http://getbootstrap.com/components/#panels
    """

    model = models.Boostrap3PanelBodyPlugin
    name = _("Panel body")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/panel_body.html"
    allow_children = True
    require_parent = True
    parent_classes = ["Bootstrap3PanelCMSPlugin"]

    fieldsets = (
        (None, {"fields": ()}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3PanelFooterCMSPlugin(CMSPluginBase):
    """
    Component - Panel: "Footer" Plugin
    http://getbootstrap.com/components/#panels-footer
    """

    model = models.Boostrap3PanelFooterPlugin
    name = _("Panel footer")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/panel_footer.html"
    allow_children = True
    require_parent = True
    parent_classes = ["Bootstrap3PanelCMSPlugin"]

    fieldsets = (
        (None, {"fields": ()}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3WellCMSPlugin(CMSPluginBase):
    """
    Component - Wells: Plugin
    http://getbootstrap.com/components/#wells
    """

    model = models.Boostrap3WellPlugin
    name = _("Well")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/well.html"
    allow_children = True

    fieldsets = (
        (None, {"fields": ("size",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3TabCMSPlugin(CMSPluginBase):
    """
    JavaScript - Tab: "Wrapper" Plugin
    http://getbootstrap.com/javascript/#tabs
    """

    model = models.Bootstrap3TabPlugin
    name = _("Tab")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/tab.html"
    allow_children = True
    # TODO add dropdown support once available as plugin
    child_classes = ["Bootstrap3TabItemCMSPlugin"]

    fieldsets = (
        (None, {"fields": ("index", ("style", "effect",),)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3TabCMSPlugin, self).render(context, instance, placeholder)
        context["tab_plugin"] = instance
        return context


class Bootstrap3TabItemCMSPlugin(CMSPluginBase):
    """
    JavaScript - Tab: "Item" Plugin
    http://getbootstrap.com/javascript/#tabs
    """

    model = models.Bootstrap3TabItemPlugin
    name = _("Tab item")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/tab_item.html"
    allow_children = True
    parent_classes = ["Bootstrap3TabCMSPlugin"]

    fieldsets = (
        (None, {"fields": ("title", "icon",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )


class Bootstrap3AccordionCMSPlugin(CMSPluginBase):
    """
    JavaScript - Collapse: "Accordion" Plugin
    http://getbootstrap.com/javascript/#collapse
    """

    model = models.Bootstrap3AccordionPlugin
    name = _("Accordion")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/accordion.html"
    allow_children = True
    child_classes = ["Bootstrap3AccordionItemCMSPlugin"]

    fieldsets = (
        (None, {"fields": ("index",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3AccordionCMSPlugin, self).render(context, instance, placeholder)
        context["accordion"] = instance
        context["accordion_id"] = "plugin-bootstrap3-accordion-{}".format(instance.pk)
        return context


class Bootstrap3AccordionItemCMSPlugin(CMSPluginBase):
    """
    JavaScript - Collapse: "Accordion item" Plugin
    http://getbootstrap.com/javascript/#collapse
    """

    model = models.Bootstrap3AccordionItemPlugin
    name = _("Accordion item")
    module = _("Accordion")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/accordion_item.html"
    allow_children = True
    parent_classes = ["Bootstrap3AccordionCMSPlugin"]

    fieldsets = (
        (None, {"fields": ("title", "context",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def render(self, context, instance, placeholder):
        context = super(Bootstrap3AccordionItemCMSPlugin, self).render(context, instance, placeholder)
        context["item"] = instance
        return context


class CarouselBase(CMSPluginBase):
    module = _("Bootstrap 3")


class CarouselSlideBase(CarouselBase):
    """
    JavaScript - Carousel: Base Plugin
    http://getbootstrap.com/javascript/#carousel
    """

    require_parent = True
    parent_classes = ["Bootstrap3CarouselCMSPlugin"]

    def render(self, context, instance, placeholder):
        # get style from parent plugin, render chosen template
        context["instance"] = instance
        context["image"] = instance.image
        return context

    def get_slide_template(self, instance, name="slide"):
        if instance.parent is None:
            style = models.Bootstrap3CarouselPlugin.STYLE_DEFAULT
        else:
            style = getattr(
                instance.parent.get_plugin_instance()[0], "style", models.Bootstrap3CarouselPlugin.STYLE_DEFAULT,
            )
        return "aldryn_bootstrap3/plugins/carousel/{}/{}.html".format(style, name)

    def get_render_template(self, context, instance, placeholder):
        return self.get_slide_template(instance=instance)


class Bootstrap3CarouselCMSPlugin(CarouselBase):
    """
    JavaScript - Carousel: "Wrapper" Plugin
    http://getbootstrap.com/javascript/#carousel
    """

    model = models.Bootstrap3CarouselPlugin
    name = _("Carousel")
    form = forms.CarouselPluginForm
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = False
    allow_children = True
    child_classes = [
        "Bootstrap3CarouselSlideCMSPlugin",
        # 'Bootstrap3CarouselSlideFolderCMSPlugin',
    ]
    fieldsets = (
        (None, {"fields": ("style", "interval", ("aspect_ratio", "transition_effect",), ("ride", "pause", "wrap",),)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def render(self, context, instance, placeholder):
        context["instance"] = instance
        if instance.child_plugin_instances:
            number_of_slides = sum(
                [
                    plugin.folder.file_count if isinstance(plugin, Bootstrap3CarouselCMSPlugin) else 1
                    for plugin in instance.child_plugin_instances
                ]
            )
        else:
            number_of_slides = 0
        context["slides"] = list(range(number_of_slides))
        return context

    def get_render_template(self, context, instance, placeholder):
        return "aldryn_bootstrap3/plugins/carousel/{}/carousel.html".format(instance.style)


class Bootstrap3CarouselSlideCMSPlugin(CarouselSlideBase):
    """
    JavaScript - Carousel: "Slide" Plugin
    http://getbootstrap.com/javascript/#carousel
    """

    model = models.Bootstrap3CarouselSlidePlugin
    name = _("Carousel slide")
    form = forms.CarouselSlidePluginForm
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    allow_children = True

    fieldsets = (
        (None, {"fields": ("image", "content",)}),
        (
            _("Link settings"),
            {
                "classes": ("collapse",),
                "fields": (
                    ("link_url", "link_page",),
                    ("link_mailto", "link_phone"),
                    ("link_anchor", "link_target"),
                    "link_file",
                    "link_text",
                ),
            },
        ),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "link_attributes",),}),
    )

    @classmethod
    def get_render_queryset(cls):
        queryset = super(Bootstrap3CarouselSlideCMSPlugin, cls).get_render_queryset()
        return queryset.select_related("link_page")


class Bootstrap3CarouselSlideFolderCMSPlugin(CarouselSlideBase):
    """
    JavaScript - Carousel: "Slide folder" Plugin
    http://getbootstrap.com/javascript/#carousel
    """

    model = models.Bootstrap3CarouselSlideFolderPlugin
    name = _("Carousel slides folder")

    def render(self, context, instance, placeholder):
        context["instance"] = instance
        context["slide_template"] = self.get_slide_template(instance=instance, name="image_slide",)
        return context

    def get_render_template(self, context, instance, placeholder):
        return self.get_slide_template(instance=instance, name="slide_folder")


class Bootstrap3SpacerCMSPlugin(CMSPluginBase):
    """
    Custom - Spacer: Plugin
    """

    model = models.Boostrap3SpacerPlugin
    name = _("Spacer")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/spacer.html"
    text_enabled = True
    text_editor_preview = False

    fieldsets = (
        (None, {"fields": ("size",)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def icon_src(self, instance):
        return static("aldryn_bootstrap3/img/type/spacer.png")


class Bootstrap3FileCMSPlugin(CMSPluginBase):
    """
    Custom - File: Plugin
    """

    model = models.Bootstrap3FilePlugin
    name = _("File")
    module = _("Bootstrap 3")
    change_form_template = "admin/aldryn_bootstrap3/base.html"
    render_template = "aldryn_bootstrap3/plugins/file.html"
    text_enabled = True

    fieldsets = (
        (None, {"fields": ("file", "name", ("open_new_window", "show_file_size",), ("icon_left", "icon_right",),)}),
        (_("Advanced settings"), {"classes": ("collapse",), "fields": ("classes", "attributes",),}),
    )

    def icon_src(self, instance):
        return static("aldryn_bootstrap3/img/type/file.png")


plugin_pool.register_plugin(Bootstrap3RowCMSPlugin)
plugin_pool.register_plugin(Bootstrap3ColumnCMSPlugin)
plugin_pool.register_plugin(Bootstrap3BlockquoteCMSPlugin)
plugin_pool.register_plugin(Bootstrap3CiteCMSPlugin)
plugin_pool.register_plugin(Bootstrap3ButtonCMSPlugin)
plugin_pool.register_plugin(Bootstrap3CodeCMSPlugin)
plugin_pool.register_plugin(Bootstrap3ImageCMSPlugin)
plugin_pool.register_plugin(Bootstrap3ResponsiveCMSPlugin)
plugin_pool.register_plugin(Bootstrap3IconCMSPlugin)
plugin_pool.register_plugin(Bootstrap3LabelCMSPlugin)
plugin_pool.register_plugin(Bootstrap3JumbotronCMSPlugin)
plugin_pool.register_plugin(Bootstrap3AlertCMSPlugin)
plugin_pool.register_plugin(Bootstrap3ListGroupCMSPlugin)
plugin_pool.register_plugin(Bootstrap3ListGroupItemCMSPlugin)
plugin_pool.register_plugin(Bootstrap3PanelCMSPlugin)
plugin_pool.register_plugin(Bootstrap3PanelHeadingCMSPlugin)
plugin_pool.register_plugin(Bootstrap3PanelBodyCMSPlugin)
plugin_pool.register_plugin(Bootstrap3PanelFooterCMSPlugin)
plugin_pool.register_plugin(Bootstrap3WellCMSPlugin)
plugin_pool.register_plugin(Bootstrap3TabCMSPlugin)
plugin_pool.register_plugin(Bootstrap3TabItemCMSPlugin)
plugin_pool.register_plugin(Bootstrap3AccordionCMSPlugin)
plugin_pool.register_plugin(Bootstrap3AccordionItemCMSPlugin)
plugin_pool.register_plugin(Bootstrap3CarouselCMSPlugin)
plugin_pool.register_plugin(Bootstrap3CarouselSlideCMSPlugin)
# plugin_pool.register_plugin(Bootstrap3CarouselSlideFolderCMSPlugin)
plugin_pool.register_plugin(Bootstrap3SpacerCMSPlugin)
plugin_pool.register_plugin(Bootstrap3FileCMSPlugin)



# -*- coding: utf-8 -*-


import django.core.exceptions
import django.forms
import django.forms.models
import django.template
import django.template.loader

from django.forms.widgets import Media, TextInput, Textarea
from django.utils.translation import ugettext_lazy as _

import cms.forms.fields
import cms.models

from djangocms_attributes_field.widgets import AttributesWidget

from . import models, constants


class RowPluginBaseForm(django.forms.models.ModelForm):
    create = django.forms.IntegerField(
        label=_("Create columns"), help_text=_("Number of columns to create in this row."), required=False, min_value=0,
    )

    class Meta:
        model = models.Bootstrap3RowPlugin
        exclude = ("page", "position", "placeholder", "language", "plugin_type")


extra_fields_row = {}
for size, name in constants.DEVICE_CHOICES:
    extra_fields_row["create_{}_col".format(size)] = django.forms.IntegerField(
        label="col-{}-".format(size),
        help_text=_("Width of created columns " "(can be edited later if required.)"),
        required=False,
        min_value=1,
        max_value=constants.GRID_SIZE,
    )
    extra_fields_row["create_{}_offset".format(size)] = django.forms.IntegerField(
        label="offset-".format(size),
        help_text=_("Offset of created columns " "(can be edited later if required.)"),
        required=False,
        min_value=0,
        max_value=constants.GRID_SIZE,
    )
    extra_fields_row["create_{}_push".format(size)] = django.forms.IntegerField(
        label="push-".format(size),
        help_text=_("Push of created columns " "(can be edited later if required.)"),
        required=False,
        min_value=0,
        max_value=constants.GRID_SIZE,
    )
    extra_fields_row["create_{}_pull".format(size)] = django.forms.IntegerField(
        label="pull-".format(size),
        help_text=_("Pull of created columns " "(can be edited later if required.)"),
        required=False,
        min_value=0,
        max_value=constants.GRID_SIZE,
    )

RowPluginForm = type(str("RowPluginBaseForm"), (RowPluginBaseForm,), extra_fields_row)


class ColumnPluginBaseForm(django.forms.models.ModelForm):
    create = django.forms.IntegerField(
        label=_("Adjust columns"), help_text=_("Adjust this column."), required=False, min_value=0,
    )

    class Meta:
        model = models.Bootstrap3ColumnPlugin
        exclude = ("page", "position", "placeholder", "language", "plugin_type")


extra_fields_column = {}
for size, name in constants.DEVICE_CHOICES:
    extra_fields_column["{}_col".format(size)] = django.forms.IntegerField(
        label="col-{}-".format(size),
        help_text=_("Width of created columns " "(can be edited later if required.)"),
        required=False,
        min_value=1,
        max_value=constants.GRID_SIZE,
    )
    extra_fields_column["{}_offset".format(size)] = django.forms.IntegerField(
        label="offset-".format(size),
        help_text=_("Offset of created columns " "(can be edited later if required.)"),
        required=False,
        min_value=0,
        max_value=constants.GRID_SIZE,
    )
    extra_fields_column["{}_push".format(size)] = django.forms.IntegerField(
        label="push-".format(size),
        help_text=_("Push of created columns " "(can be edited later if required.)"),
        required=False,
        min_value=0,
        max_value=constants.GRID_SIZE,
    )
    extra_fields_column["{}_pull".format(size)] = django.forms.IntegerField(
        label="pull-".format(size),
        help_text=_("Pull of created columns " "(can be edited later if required.)"),
        required=False,
        min_value=0,
        max_value=constants.GRID_SIZE,
    )

ColumnPluginForm = type(str("ColumnPluginBaseForm"), (ColumnPluginBaseForm,), extra_fields_column)


class Bootstrap3CodePluginForm(django.forms.models.ModelForm):
    class Meta:
        # When used inside djangocms-text-ckeditor
        # this causes the label field to be prefilled with the selected text.
        widgets = {
            "code": Textarea(attrs={"class": "js-ckeditor-use-selected-text"}),
        }


class LinkForm(django.forms.models.ModelForm):
    link_page = cms.forms.fields.PageSelectFormField(
        queryset=cms.models.Page.objects.drafts(), label=_("Internal link"), required=False,
    )

    def for_site(self, site):
        # override the page_link fields queryset to containt just pages for
        # current site
        self.fields["link_page"].queryset = cms.models.Page.objects.drafts().on_site(site)

    class Meta:
        model = models.Boostrap3ButtonPlugin
        exclude = (
            "page",
            "position",
            "placeholder",
            "language",
            "plugin_type",
        )
        # When used inside djangocms-text-ckeditor
        # this causes the label field to be prefilled with the selected text.
        widgets = {
            "label": TextInput(attrs={"class": "js-ckeditor-use-selected-text"}),
        }

    def __init__(self, *args, **kwargs):
        super(LinkForm, self).__init__(*args, **kwargs)
        self.fields["link_attributes"].widget = AttributesWidget()

    def _get_media(self):
        """
        Provide a description of all media required to render the widgets on this form
        """
        media = Media()
        for field in list(self.fields.values()):
            media = media + field.widget.media
        media._js = ["cms/js/libs/jquery.min.js"] + media._js
        return media

    media = property(_get_media)


class Boostrap3LabelPluginForm(django.forms.models.ModelForm):
    class Meta:
        model = models.Boostrap3LabelPlugin
        exclude = ("page", "position", "placeholder", "language", "plugin_type")
        # When used inside djangocms-text-ckeditor
        # this causes the label field to be prefilled with the selected text.
        widgets = {
            "label": TextInput(attrs={"class": "js-ckeditor-use-selected-text"}),
        }


class PanelPluginBaseForm(django.forms.models.ModelForm):
    create_heading = django.forms.BooleanField(label=_("Initial heading"), required=False, initial=False,)
    create_body = django.forms.BooleanField(label=_("Initial body"), required=False, initial=False,)
    create_footer = django.forms.BooleanField(label=_("Initial footer"), required=False, initial=False,)

    class Meta:
        model = models.Boostrap3PanelPlugin
        exclude = ("page", "position", "placeholder", "language", "plugin_type")


class CarouselPluginForm(django.forms.ModelForm):
    class Meta:
        fields = [
            "style",
            "transition_effect",
            "ride",
            "interval",
        ]
        model = models.Bootstrap3CarouselPlugin

    def clean_style(self):
        style = self.cleaned_data.get("style")
        template = "aldryn_bootstrap3/plugins/carousel/{}/carousel.html".format(style)
        # Check if template for style exists:
        try:
            django.template.loader.select_template([template])
        except django.template.TemplateDoesNotExist:
            raise django.forms.ValidationError(
                _("Not a valid style (template {path} does not exist)").format(path=template)
            )
        return style


class CarouselSlidePluginForm(django.forms.ModelForm):
    class Meta:
        fields = ["image", "content", "link_text", "classes", "link_attributes"]
        model = models.Bootstrap3CarouselSlidePlugin

    def __init__(self, *args, **kwargs):
        super(CarouselSlidePluginForm, self).__init__(*args, **kwargs)
        self.fields["link_attributes"].widget = AttributesWidget()



# -*- coding: utf-8 -*-


import collections
import os
from functools import partial

import django.forms.models
import djangocms_text_ckeditor.fields
import filer.fields.folder
from cms.models.pluginmodel import CMSPlugin
from django.db import models
from django.utils.html import strip_tags
from django.utils.translation import ugettext, ugettext_lazy as _, ungettext
from djangocms_attributes_field.fields import AttributesField

from . import model_fields, constants


# CSS - http://getbootstrap.com/css/
# Global CSS settings, fundamental HTML elements styled and enhanced with
# extensible classes, and an advanced grid system.
#
# The following components marked with "✓" are implemented:
#
# [✓] Grid
# [✓] Typography (Blockquote and Cite)
# [✓] Code
# [✓] Forms (via aldryn-forms)
# [✓] Buttons
# [✓] Images
# [✓] Helper classes (js/ckeditor.js)
# [✓] Responsive utilities


class Bootstrap3RowPlugin(CMSPlugin):
    """
    CSS - Grid system: "Row" Model
    http://getbootstrap.com/css/#grid
    """

    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        instance = self.get_plugin_instance()[0]

        if not instance:
            return ugettext("<empty>")

        column_count = len(self.child_plugin_instances or [])
        column_count_str = ungettext("1 column", "%(count)i columns", column_count) % {"count": column_count}

        if self.classes:
            return "{} ({})".format(self.classes, column_count_str)
        return column_count_str


class Bootstrap3ColumnPlugin(CMSPlugin):
    """
    CSS - Grid system: "Column" Model
    http://getbootstrap.com/css/#grid
    """

    tag = models.SlugField(verbose_name=_("Tag"), default="div",)
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        txt = " ".join([self.get_column_classes(), self.classes])
        if self.tag != "div":
            txt = "{} ({})".format(txt, self.tag)
        return txt

    def get_class(self, device, element):
        size = getattr(self, "{}_{}".format(device, element), None)
        if size is not None:
            if element == "col":
                return "col-{}-{}".format(device, size)
            else:
                return "col-{}-{}-{}".format(device, element, size)
        return ""

    def get_column_classes(self):
        classes = []
        for device in constants.DEVICE_SIZES:
            for element in ("col", "offset", "push", "pull"):
                classes.append(self.get_class(device, element))
        return " ".join(cls for cls in classes if cls)


ColSizeField = partial(
    model_fields.IntegerField, null=True, blank=True, default=None, min_value=1, max_value=constants.GRID_SIZE,
)

OffsetSizeField = partial(
    model_fields.IntegerField, null=True, blank=True, default=None, min_value=0, max_value=constants.GRID_SIZE,
)

PushSizeField = partial(
    model_fields.IntegerField, null=True, blank=True, default=None, min_value=0, max_value=constants.GRID_SIZE,
)

PullSizeField = partial(
    model_fields.IntegerField, null=True, blank=True, default=None, min_value=0, max_value=constants.GRID_SIZE,
)

for size, name in constants.DEVICE_CHOICES:
    Bootstrap3ColumnPlugin.add_to_class(
        "{}_col".format(size), ColSizeField(verbose_name="col-{}-".format(size)),
    )
    Bootstrap3ColumnPlugin.add_to_class(
        "{}_offset".format(size), OffsetSizeField(verbose_name="offset-".format(size)),
    )
    Bootstrap3ColumnPlugin.add_to_class(
        "{}_push".format(size), PushSizeField(verbose_name="push-".format(size)),
    )
    Bootstrap3ColumnPlugin.add_to_class(
        "{}_pull".format(size), PullSizeField(verbose_name="pull-".format(size)),
    )


class Boostrap3BlockquotePlugin(CMSPlugin):
    """
    CSS - Typography: "Blockquote" Model
    http://getbootstrap.com/css/#type-blockquotes
    """

    reverse = models.BooleanField(
        verbose_name=_("Reverse quote"),
        default=False,
        blank=True,
        help_text=_("Reverses the position by adding the Bootstrap 3 " '"blockquote-reverse" class.'),
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        if self.reverse:
            return ".blockquote-reverse"
        return ""


class Boostrap3CitePlugin(CMSPlugin):
    """
    CSS - Typography: "Cite" Model
    http://getbootstrap.com/css/#type-blockquotes
    """

    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return ""


class Bootstrap3CodePlugin(CMSPlugin):
    """
    CSS - Code: Model
    http://getbootstrap.com/css/#code
    """

    CODE_TYPE_CHOICES = (
        ("code", _("Inline")),
        ("kbd", _("User input")),
        ("pre", _("Basic block")),
        ("var", _("Variables")),
        ("samp", _("Sample output")),
    )

    code_type = models.CharField(
        verbose_name=_("Code type"), choices=CODE_TYPE_CHOICES, default=CODE_TYPE_CHOICES[0][0], max_length=255,
    )
    code = models.TextField(verbose_name=_("Code"), blank=True,)
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return "<{}>".format(self.code_type)


class Boostrap3ButtonPlugin(CMSPlugin, model_fields.LinkMixin):
    """
    CSS - Buttons: "Button/Link" Model
    http://getbootstrap.com/css/#buttons
    """

    label = models.CharField(verbose_name=_("Display name"), blank=True, default="", max_length=255,)
    type = model_fields.LinkOrButton(verbose_name=_("Type"),)
    # button specific fields
    btn_context = model_fields.Context(
        verbose_name=_("Context"), choices=constants.BUTTON_CONTEXT_CHOICES, default=constants.BUTTON_CONTEXT_DEFAULT,
    )
    btn_size = model_fields.Size(verbose_name=_("Size"),)
    btn_block = models.BooleanField(verbose_name=_("Block"), default=False,)
    # text link specific fields
    txt_context = model_fields.Context(
        verbose_name=_("Context"),
        choices=constants.TEXT_LINK_CONTEXT_CHOICES,
        default=constants.TEXT_LINK_CONTEXT_DEFAULT,
        blank=True,
    )
    # common fields
    icon_left = model_fields.Icon(verbose_name=_("Icon left"),)
    icon_right = model_fields.Icon(verbose_name=_("Icon right"),)
    classes = model_fields.Classes()

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.label


class Boostrap3ImagePlugin(CMSPlugin):
    """
    CSS - Images: Model
    http://getbootstrap.com/css/#images
    """

    file = filer.fields.image.FilerImageField(
        verbose_name=_("Image"), blank=False, null=True, on_delete=models.SET_NULL, related_name="+",
    )
    alt = model_fields.MiniText(verbose_name=_("Alternative text"), blank=True, default="",)
    title = model_fields.MiniText(verbose_name=_("Title"), blank=True, default="",)
    use_original_image = models.BooleanField(
        verbose_name=_("Use original image"),
        blank=True,
        default=False,
        help_text=_("Outputs the raw image without cropping."),
    )
    override_width = models.IntegerField(
        verbose_name=_("Override width"),
        blank=True,
        null=True,
        help_text=_("The image width as number in pixels. " 'Example: "720" and not "720px".'),
    )
    override_height = models.IntegerField(
        verbose_name=_("Override height"),
        blank=True,
        null=True,
        help_text=_("The image height as number in pixels. " 'Example: "720" and not "720px".'),
    )
    aspect_ratio = models.CharField(
        verbose_name=_("Aspect ratio"),
        choices=constants.ASPECT_RATIO_CHOICES,
        blank=True,
        default="",
        max_length=255,
        help_text=_("Determines width and height of the image " "according to the selected ratio."),
    )
    shape = models.CharField(
        verbose_name=_("Shape"),
        choices=(("rounded", ".img-rounded"), ("circle", ".img-circle"),),
        default="",
        blank=True,
        max_length=255,
    )
    thumbnail = models.BooleanField(
        verbose_name=".img-thumbnail",
        default=False,
        blank=True,
        help_text=_('Adds the Bootstrap 3 ".img-thumbnail" class.'),
    )
    img_responsive = models.BooleanField(
        verbose_name=".img-responsive",
        default=True,
        blank=True,
        help_text=_('Adds the Bootstrap 3 ".img-responsive" class.'),
    )
    attributes = AttributesField(
        verbose_name=_("Attributes"), blank=True, excluded_keys=["src", "alt", "class", "title"],
    )
    classes = model_fields.Classes()

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        txt = ""
        if self.file_id and self.file.label:
            txt = self.file.label
        return txt

    def srcset(self):
        if not self.file:
            return []
        items = collections.OrderedDict()
        if self.aspect_ratio:
            aspect_width, aspect_height = tuple([int(i) for i in self.aspect_ratio.split("x")])
        else:
            aspect_width, aspect_height = None, None
        for device in constants.DEVICES:
            if self.override_width:
                width = self.override_width
            else:
                # TODO: should this should be based on the containing col size?
                width = device["width_gutter"]
            width_tag = str(width)
            if aspect_width is not None and aspect_height is not None:
                height = int(float(width) * float(aspect_height) / float(aspect_width))
                crop = True
            else:
                if self.override_height:
                    height = self.override_height
                else:
                    height = 0
                crop = False
            items[device["identifier"]] = {
                "size": (width, height),
                "size_str": "{}x{}".format(width, height),
                "width_str": "{}w".format(width),
                "subject_location": self.file.subject_location,
                "upscale": True,
                "crop": crop,
                "aspect_ratio": (aspect_width, aspect_height),
                "width_tag": width_tag,
            }

        return items


class Bootstrap3ResponsivePlugin(CMSPlugin):
    """
    CSS - Responsive: "Utilities" Model
    http://getbootstrap.com/css/#responsive-utilities
    """

    device_breakpoints = model_fields.Responsive(
        verbose_name=_("Devices"), default="visible-xs visible-sm visible-md visible-lg",
    )
    print_breakpoints = model_fields.ResponsivePrint(verbose_name=_("Print"), default="visible-print",)
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        text = self.device_breakpoints
        if self.print_breakpoints:
            text += " {}".format(self.print_breakpoints)
        return text


# Components - http://getbootstrap.com/components/
# Over a dozen reusable components built to provide iconography, dropdowns,
# input groups, navigation, alerts, and much more.
#
# The following components marked with "✓" are implemented:
#
# [✓] Glyphicons
# [ ] Dropdowns
# [ ] Button Groups
# [ ] Button Dropdowns
# [✓] Input Groups (via aldryn-forms)
# [✗] Navs (not applicable)
# [✗] Navbar (not applicable)
# [✗] Breadcrumbs (not applicable)
# [✗] Pagination (not applicable)
# [✓] Labels
# [✗] Badges (not applicable)
# [✓] Jumbotron
# [ ] Page header
# [ ] Thumbnails
# [✓] Alerts
# [ ] Progress Bars
# [ ] Media object
# [✓] List Group
# [✓] Panels
# [ ] Responsive embed
# [✓] Wells


class Boostrap3IconPlugin(CMSPlugin):
    """
    Component - Glyphicons: "Icon" Model
    http://getbootstrap.com/components/#glyphicons
    Component - Font Awesome: "Icon" Model
    http://fontawesome.io/
    """

    icon = model_fields.Icon(verbose_name=_("Icon"), blank=False,)
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.icon


class Boostrap3LabelPlugin(CMSPlugin):
    """
    Component - Label: Model
    http://getbootstrap.com/components/#labels
    """

    label = models.CharField(verbose_name=_("Label"), blank=True, default="", max_length=255,)
    context = model_fields.Context(
        verbose_name=_("Context"),
        choices=(("default", _("Default")),) + constants.CONTEXT_CHOICES,
        default="default",
        blank=False,
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.label


class Boostrap3JumbotronPlugin(CMSPlugin):
    """
    Component - Jumbotron: Model
    http://getbootstrap.com/components/#jumbotron
    """

    label = models.CharField(verbose_name=_("Label"), blank=True, max_length=255,)
    grid = models.BooleanField(
        verbose_name=_("Add container"),
        default=False,
        blank=True,
        help_text=_('Adds a ".container" element inside the "Jumbotron" ' "for use outside of a grid."),
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.label or str(self.pk)


class Boostrap3AlertPlugin(CMSPlugin):
    """
    Component - Alert: Model
    http://getbootstrap.com/components/#alerts
    """

    context = model_fields.Context(verbose_name=_("Context"),)
    icon = model_fields.Icon(verbose_name=_("Title icon"),)
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.classes


class Bootstrap3ListGroupPlugin(CMSPlugin):
    """
    Component - List group: "Wrapper" Model
    http://getbootstrap.com/components/#alerts
    """

    add_list_group_class = models.BooleanField(
        verbose_name=".list-group",
        default=True,
        blank=True,
        help_text=_("Adds the list-group and subsequent list-group-item classes."),
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def get_short_description(self):
        instance = self.get_plugin_instance()[0]

        if not instance:
            return ugettext("<empty>")

        column_count = len(self.child_plugin_instances or [])
        column_count_str = ungettext("1 item", "%(count)i items", column_count) % {"count": column_count}
        return column_count_str


class Bootstrap3ListGroupItemPlugin(CMSPlugin):
    """
    Component - List group: "Item" Model
    http://getbootstrap.com/components/#alerts
    """

    title = model_fields.MiniText(verbose_name=_("Title"), blank=True, default="",)
    context = model_fields.Context(
        verbose_name=_("Context"),
        choices=(("default", _("Default")),) + constants.CONTEXT_CHOICES,
        default="default",
        blank=False,
    )
    state = models.CharField(
        verbose_name=_("State"),
        choices=(("active", _("Active")), ("disabled", _("Disabled")),),
        blank=True,
        max_length=255,
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.title


class Boostrap3PanelPlugin(CMSPlugin):
    """
    Component - Panel: "Wrapper" Model
    http://getbootstrap.com/components/#panels
    """

    context = model_fields.Context(
        verbose_name=_("Context"),
        choices=(("default", _("Default")),) + constants.CONTEXT_CHOICES,
        default="default",
        blank=False,
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.context


class Boostrap3PanelHeadingPlugin(CMSPlugin):
    """
    Component - Panel: "Heading" Model
    http://getbootstrap.com/components/#panels-heading
    """

    title = model_fields.MiniText(
        verbose_name=_("Title"), blank=True, default="", help_text=_("Panels can have additional plugins.")
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.title


class Boostrap3PanelBodyPlugin(CMSPlugin):
    """
    Component - Panel: "Body" Model
    http://getbootstrap.com/components/#panels
    """

    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.classes


class Boostrap3PanelFooterPlugin(CMSPlugin):
    """
    Component - Panel: "Footer" Model
    http://getbootstrap.com/components/#panels-footer
    """

    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.classes


class Boostrap3WellPlugin(CMSPlugin):
    """
    Component - Wells: Model
    http://getbootstrap.com/components/#wells
    """

    size = model_fields.Size(verbose_name=_("Size"),)
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.classes


# JavaScript - http://getbootstrap.com/javascript/
# Bring Bootstrap's components to life with over a dozen custom jQuery plugins.
# Easily include them all, or one by one.
#
# The following components marked with "✓" are implemented:
#
# [x] Transitions (not applicable)
# [ ] Modal
# [ ] Dropdowns
# [ ] Scrollspy
# [✓] Tab
# [ ] Tooltip
# [ ] Popover
# [ ] Alert
# [ ] Button
# [✓] Collapse
# [✓] Carousel
# [ ] Affix


class Bootstrap3TabPlugin(CMSPlugin):
    """
    JavaScript - Tab: "Wrapper" Model
    http://getbootstrap.com/javascript/#tabs
    """

    TAB_TYPE_CHOICES = (
        ("nav-tabs", _("Tabs")),
        ("nav-tabs nav-justified", _("Tabs justified")),
        ("nav-pills", _("Pills")),
        ("nav-pills nav-justified", _("Pills justified")),
    )
    TAB_EFFECT_CHOICES = (("fade", _("Fade")),)

    index = models.PositiveIntegerField(
        verbose_name=_("Index"),
        null=True,
        blank=True,
        help_text=_("Index of element to open on page load (optional)."),
    )
    style = models.CharField(
        verbose_name=_("Display type"), choices=TAB_TYPE_CHOICES, default=TAB_TYPE_CHOICES[0][0], max_length=255,
    )
    effect = models.CharField(
        verbose_name=_("Animation effect"), choices=TAB_EFFECT_CHOICES, blank=True, max_length=255,
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return "{} {}".format(self.style, self.effect)


class Bootstrap3TabItemPlugin(CMSPlugin):
    """
    JavaScript - Tab: "Item" Model
    http://getbootstrap.com/javascript/#tabs
    """

    title = models.CharField(verbose_name=_("Tab title"), max_length=255,)
    icon = model_fields.Icon(verbose_name=_("Title icon"),)
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.title


class Bootstrap3AccordionPlugin(CMSPlugin):
    """
    JavaScript - Collapse: "Accordion" Model
    http://getbootstrap.com/javascript/#collapse
    """

    index = models.PositiveIntegerField(
        verbose_name=_("Index"),
        null=True,
        blank=True,
        help_text=_("Index of element to open on page load (optional)."),
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def get_short_description(self):
        instance = self.get_plugin_instance()[0]

        if not instance:
            return ugettext("<empty>")

        column_count = len(self.child_plugin_instances or [])
        column_count_str = ungettext("1 item", "%(count)i items", column_count) % {"count": column_count}
        return column_count_str

    def __str__(self):
        return str(self.index)


class Bootstrap3AccordionItemPlugin(CMSPlugin):
    """
    JavaScript - Collapse: "Accordion item" Model
    http://getbootstrap.com/javascript/#collapse
    """

    title = model_fields.MiniText(verbose_name=_("Title"), default="", blank=True,)
    context = model_fields.Context(
        verbose_name=_("Context"),
        choices=(("default", _("Default")),) + constants.CONTEXT_CHOICES,
        default="default",
        blank=False,
    )
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return self.title


class Bootstrap3CarouselPlugin(CMSPlugin):
    """
    JavaScript - Carousel: "Wrapper" Model
    http://getbootstrap.com/javascript/#carousel
    """

    STYLE_DEFAULT = "standard"
    STYLE_CHOICES = [
        (STYLE_DEFAULT, _("Standard")),
    ]
    TRANSITION_EFFECT_CHOICES = (("slide", _("Slide")),)

    style = models.CharField(
        verbose_name=_("Style"),
        choices=STYLE_CHOICES + model_fields.get_additional_styles(),
        default=STYLE_DEFAULT,
        max_length=255,
    )
    aspect_ratio = models.CharField(
        verbose_name=_("Aspect ratio"),
        choices=constants.ASPECT_RATIO_CHOICES,
        blank=True,
        default="",
        max_length=255,
        help_text=_("Determines width and height of the image " "according to the selected ratio."),
    )
    transition_effect = models.CharField(
        verbose_name=_("Transition effect"), choices=TRANSITION_EFFECT_CHOICES, default="", blank=True, max_length=255,
    )
    ride = models.BooleanField(
        verbose_name=_("Ride"), default=True, help_text=_("Auto-starts animation of the carousel."),
    )
    interval = models.IntegerField(
        verbose_name=_("Interval"), default=5000, help_text=_("Time (in milliseconds) between items."),
    )
    wrap = models.BooleanField(
        verbose_name=_("Wrap"), default=True, blank=True, help_text=_("Loops carousel animation."),
    )
    pause = models.BooleanField(
        verbose_name=_("Pause"), default=True, blank=True, help_text=_("Pauses the carousel on hover."),
    )
    classes = model_fields.Classes()
    attributes = AttributesField(
        verbose_name=_("Attributes"),
        blank=True,
        excluded_keys=[
            "class",
            # data attributes et via settings
            "data-ride",
            "data-interval",
            "data-pause",
            "data-wrap",
        ],
    )

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        data = django.forms.models.model_to_dict(self)
        data.update(
            dict(
                style_label=_("Style"),
                transition_effect_label=_("Transition effect"),
                ride_label=_("Ride"),
                interval_label=_("Interval"),
                aspect_ratio_label=_("Aspect ratio"),
            )
        )
        fields = [
            "style",
            "transition_effect",
            "ride",
            "interval",
            "aspect_ratio",
        ]
        if not data["ride"]:
            fields.remove("interval")
        return ", ".join(
            ["{key}: {value}".format(key=data["{}_label".format(field)], value=data[field]) for field in fields]
        )

    def srcset(self):
        # more or less copied from image plugin.
        # TODO: replace with generic sizes/srcset solution
        items = collections.OrderedDict()
        if self.aspect_ratio:
            aspect_width, aspect_height = tuple([int(i) for i in self.aspect_ratio.split("x")])
        else:
            aspect_width, aspect_height = None, None
        for device in constants.DEVICES:
            width = device["width_gutter"]  # TODO: should this should be based on the containing col size?
            width_tag = str(width)
            if aspect_width is not None and aspect_height is not None:
                height = int(float(width) * float(aspect_height) / float(aspect_width))
                crop = True
            else:
                height = 0
                crop = False
            items[device["identifier"]] = {
                "size": (width, height),
                "size_str": "{}x{}".format(width, height),
                "width_str": "{}w".format(width),
                # 'subject_location': self.file.subject_location,
                "upscale": True,
                "crop": crop,
                "aspect_ratio": (aspect_width, aspect_height),
                "width_tag": width_tag,
            }

        return items


class Bootstrap3CarouselSlidePlugin(CMSPlugin, model_fields.LinkMixin):
    """
    JavaScript - Carousel: "Slide" Model
    http://getbootstrap.com/javascript/#carousel
    """

    image = filer.fields.image.FilerImageField(
        verbose_name=_("Image"), blank=False, null=True, on_delete=models.SET_NULL, related_name="+",
    )
    link_text = models.CharField(verbose_name=_("Link text"), max_length=255, blank=True,)
    content = djangocms_text_ckeditor.fields.HTMLField(
        verbose_name=_("Content"),
        blank=True,
        default="",
        help_text=_("Content may also be added using child plugins."),
    )
    classes = model_fields.Classes()

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        image_text = content_text = ""

        if self.image_id:
            if self.image.name:
                image_text = self.image.name
            elif self.image.original_filename and os.path.split(self.image.original_filename)[1]:
                image_text = os.path.split(self.image.original_filename)[1]
            else:
                image_text = "Image"
        if self.content:
            text = strip_tags(self.content).strip()
            if len(text) > 100:
                content_text = "{}...".format(text[:100])
            else:
                content_text = "{}".format(text)

        if image_text and content_text:
            return "{} ({})".format(image_text, content_text)
        else:
            return image_text or content_text


class Bootstrap3CarouselSlideFolderPlugin(CMSPlugin):
    """
    JavaScript - Carousel: "Slide folder" Model
    http://getbootstrap.com/javascript/#carousel
    """

    folder = filer.fields.folder.FilerFolderField(verbose_name=_("Folder"), on_delete=models.CASCADE)
    classes = model_fields.Classes()

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        if self.folder_id:
            return self.folder.pretty_logical_path
        else:
            return _("<folder is missing>")


# Custom plugins added to support further stylings
#
# [✓] Spacer
# [✓] File


class Boostrap3SpacerPlugin(CMSPlugin):
    """
    Custom - Spacer: Model
    """

    size = model_fields.Size(verbose_name=_("Size"),)
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        return "size-" + self.size + " " + self.classes


class Bootstrap3FilePlugin(CMSPlugin):
    """
    Custom - File: Model
    """

    file = filer.fields.file.FilerFileField(
        verbose_name=_("File"), null=True, blank=False, on_delete=models.SET_NULL, related_name="+",
    )
    name = model_fields.MiniText(verbose_name=_("Name"), default="", blank=True,)
    open_new_window = models.BooleanField(verbose_name=_("Open in new window"), default=False,)
    show_file_size = models.BooleanField(verbose_name=_("Show file size"), default=False,)
    # common fields
    icon_left = model_fields.Icon()
    icon_right = model_fields.Icon()
    # additional fields
    classes = model_fields.Classes()
    attributes = AttributesField(verbose_name=_("Attributes"), blank=True, excluded_keys=["class", "target"],)

    cmsplugin_ptr = model_fields.CMSPluginField()

    def __str__(self):
        label = self.name
        if not label:
            if self.file_id:
                label = self.file.label
            else:
                label = "File"
        return label



from django.forms.utils import flatatt
from django.utils.encoding import force_text
from django.utils.html import html_safe, format_html
from django.utils.safestring import mark_safe


@html_safe
class SubWidget(object):
    """
    This class is only needed for Django >= 1.11 compatibility

    Some widgets are made of multiple HTML elements -- namely, RadioSelect.
    This is a class that represents the "inner" HTML element of a widget.
    """

    def __init__(self, parent_widget, name, value, attrs, choices):
        self.parent_widget = parent_widget
        self.name, self.value = name, value
        self.attrs, self.choices = attrs, choices

    def __str__(self):
        args = [self.name, self.value, self.attrs]
        if self.choices:
            args.append(self.choices)
        return self.parent_widget.render(*args)


@html_safe
class ChoiceInput(SubWidget):
    """
    This class is only needed for Django >= 1.11 compatibility

    An object used by ChoiceFieldRenderer that represents a single
    <input type='$input_type'>.
    """

    input_type = None  # Subclasses must define this

    def __init__(self, name, value, attrs, choice, index):
        self.name = name
        self.value = value
        self.attrs = attrs
        self.choice_value = force_text(choice[0])
        self.choice_label = force_text(choice[1])
        self.index = index
        if "id" in self.attrs:
            self.attrs["id"] += "_%d" % self.index

    def __str__(self):
        return self.render()

    def render(self, name=None, value=None, attrs=None):
        if self.id_for_label:
            label_for = format_html(' for="{}"', self.id_for_label)
        else:
            label_for = ""
        attrs = dict(self.attrs, **attrs) if attrs else self.attrs
        return format_html("<label{}>{} {}</label>", label_for, self.tag(attrs), self.choice_label)

    def is_checked(self):
        return self.value == self.choice_value

    def tag(self, attrs=None):
        attrs = attrs or self.attrs
        final_attrs = dict(attrs, type=self.input_type, name=self.name, value=self.choice_value)
        if self.is_checked():
            final_attrs["checked"] = "checked"
        return format_html("<input{} />", flatatt(final_attrs))

    @property
    def id_for_label(self):
        return self.attrs.get("id", "")


class CheckboxChoiceInput(ChoiceInput):
    """
    This class is only needed for Django >= 1.11 compatibility
    """

    input_type = "checkbox"

    def __init__(self, *args, **kwargs):
        super(CheckboxChoiceInput, self).__init__(*args, **kwargs)
        self.value = set(force_text(v) for v in self.value)

    def is_checked(self):
        return self.choice_value in self.value


class RadioChoiceInput(ChoiceInput):
    input_type = "radio"

    def __init__(self, *args, **kwargs):
        super(RadioChoiceInput, self).__init__(*args, **kwargs)
        self.value = force_text(self.value)


@html_safe
class ChoiceFieldRenderer(object):
    """
    This class is only needed for Django >= 1.11 compatibility

    An object used by RadioSelect to enable customization of radio widgets.
    """

    choice_input_class = None
    outer_html = "<ul{id_attr}>{content}</ul>"
    inner_html = "<li>{choice_value}{sub_widgets}</li>"

    def __init__(self, name, value, attrs, choices):
        self.name = name
        self.value = value
        self.attrs = attrs
        self.choices = choices

    def __getitem__(self, idx):
        return list(self)[idx]

    def __iter__(self):
        for idx, choice in enumerate(self.choices):
            yield self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, idx)

    def __str__(self):
        return self.render()

    def render(self):
        """
        This class is only needed for Django >= 1.11 compatibility

        Outputs a <ul> for this set of choice fields.
        If an id was given to the field, it is applied to the <ul> (each
        item in the list will get an id of `$id_$i`).
        """
        id_ = self.attrs.get("id")
        output = []
        for i, choice in enumerate(self.choices):
            choice_value, choice_label = choice
            if isinstance(choice_label, (tuple, list)):
                attrs_plus = self.attrs.copy()
                if id_:
                    attrs_plus["id"] += "_{}".format(i)
                sub_ul_renderer = self.__class__(
                    name=self.name, value=self.value, attrs=attrs_plus, choices=choice_label
                )
                sub_ul_renderer.choice_input_class = self.choice_input_class
                output.append(
                    format_html(self.inner_html, choice_value=choice_value, sub_widgets=sub_ul_renderer.render())
                )
            else:
                w = self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, i)
                output.append(format_html(self.inner_html, choice_value=force_text(w), sub_widgets=""))
        return format_html(
            self.outer_html, id_attr=format_html(' id="{}"', id_) if id_ else "", content=mark_safe("\n".join(output))
        )


class RadioFieldRenderer(ChoiceFieldRenderer):
    """
    This class is only needed for Django >= 1.11 compatibility
    """

    choice_input_class = RadioChoiceInput


class CheckboxFieldRenderer(ChoiceFieldRenderer):
    """
    This class is only needed for Django >= 1.11 compatibility
    """

    choice_input_class = CheckboxChoiceInput


class RendererMixin(object):
    """
    This class is only needed for Django >= 1.11 compatibility
    """

    renderer = None  # subclasses must define this
    _empty_value = None

    def __init__(self, *args, **kwargs):
        # Override the default renderer if we were passed one.
        renderer = kwargs.pop("renderer", None)
        if renderer:
            self.renderer = renderer
        super(RendererMixin, self).__init__(*args, **kwargs)

    def subwidgets(self, name, value, attrs=None):
        for widget in self.get_renderer(name, value, attrs):
            yield widget

    def get_renderer(self, name, value, attrs=None):
        """Returns an instance of the renderer."""
        if value is None:
            value = self._empty_value
        final_attrs = self.build_attrs(attrs)
        return self.renderer(name, value, final_attrs, self.choices)

    def render(self, name, value, attrs=None):
        return self.get_renderer(name, value, attrs).render()

    def id_for_label(self, id_):
        # Widgets using this RendererMixin are made of a collection of
        # subwidgets, each with their own <label>, and distinct ID.
        # The IDs are made distinct by a "_X" suffix, where X is the zero-based
        # index of the choice field. Thus, the label for the main widget should
        # reference the first subwidget, hence the "_0" suffix.
        if id_:
            id_ += "_0"
        return id_



# -*- coding: utf-8 -*-


from django.utils.translation import ugettext_lazy as _

from .conf import settings


# Changable constants, overriden through settings
GRID_SIZE = getattr(settings, "ALDRYN_BOOTSTRAP3_GRID_SIZE", 24)

# Fixed constants, not influenced by settings
# Changes here will most likely require database migrtions
DEVICE_CHOICES = (
    ("xs", _("Tiny")),
    ("sm", _("Small")),
    ("md", _("Medium")),
    ("lg", _("Large")),
)

DEVICE_SIZES = tuple([size for size, name in DEVICE_CHOICES])

TARGET_CHOICES = (
    ("_blank", _("Open in new window")),
    ("_self", _("Open in same window")),
    ("_parent", _("Delegate to parent")),
    ("_top", _("Delegate to top")),
)

CONTEXT_CHOICES = (
    ("primary", _("Primary"),),
    ("success", _("Success"),),
    ("info", _("Info"),),
    ("warning", _("Warning"),),
    ("danger", _("Danger"),),
)

CONTEXT_DEFAULT = "default"

BUTTON_CONTEXT_CHOICES = (("default", _("Default"),),) + CONTEXT_CHOICES + (("link", _("Link"),),)

BUTTON_CONTEXT_DEFAULT = "default"

TEXT_LINK_CONTEXT_CHOICES = (("", _("Default"),),) + CONTEXT_CHOICES + (("muted ", _("Muted"),),)

TEXT_LINK_CONTEXT_DEFAULT = ""

ASPECT_RATIOS = (
    (4, 3),
    (16, 9),
    (16, 10),
    (21, 9),
)

ASPECT_RATIOS_REVERSED = [(y, x) for x, y in ASPECT_RATIOS]

ASPECT_RATIO_CHOICES = (
    tuple([("{0}x{1}".format(1, 1), "{0}x{1}".format(1, 1))])
    + tuple([("{0}x{1}".format(x, y), "{0}x{1}".format(x, y)) for x, y in ASPECT_RATIOS])
    + tuple([("{0}x{1}".format(x, y), "{0}x{1}".format(x, y)) for x, y in ASPECT_RATIOS_REVERSED])
)

SIZE_CHOICES = (
    ("lg", _("Large"),),
    ("md", _("Medium"),),
    ("sm", _("Small"),),
    ("xs", _("Extra Small"),),
)

DEVICES = (
    {"identifier": "xs", "name": _("Mobile phones"), "width": 768, "width_gutter": 750, "icon": "mobile-phone",},
    {"identifier": "sm", "name": _("Tablets"), "width": 768, "width_gutter": 750, "icon": "tablet",},
    {"identifier": "md", "name": _("Laptops"), "width": 992, "width_gutter": 970, "icon": "laptop",},
    {"identifier": "lg", "name": _("Large desktops"), "width": 1200, "width_gutter": 1170, "icon": "desktop",},
)

for device in DEVICES:
    identifier = device["identifier"]
    device["long_description"] = "{name} (<{width}px)".format(**device)
    device["size_name"] = dict(SIZE_CHOICES).get(identifier)



# -*- coding: utf-8 -*-


import cms.models.fields
import django.db.models.deletion
import djangocms_text_ckeditor.fields
import filer.fields.file
import filer.fields.folder
import filer.fields.image
from django.db import models, migrations

import aldryn_bootstrap3.model_fields


class Migration(migrations.Migration):
    dependencies = [
        ("filer", "0002_auto_20150606_2003"),
        ("cms", "0011_auto_20150419_1006"),
    ]

    operations = [
        migrations.CreateModel(
            name="Boostrap3AlertPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("context", aldryn_bootstrap3.model_fields.Context(default="default", max_length=255)),
                ("icon", aldryn_bootstrap3.model_fields.Icon(default="", max_length=255, blank=True)),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3BlockquotePlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("reverse", models.BooleanField(default=False)),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3ButtonPlugin",
            fields=[
                ("link_url", models.URLField(default="", verbose_name="link", blank=True)),
                (
                    "link_anchor",
                    models.CharField(
                        help_text="Adds this value as an anchor (#my-anchor) to the link.",
                        max_length=128,
                        verbose_name="anchor",
                        blank=True,
                    ),
                ),
                ("link_mailto", models.EmailField(max_length=75, null=True, verbose_name="mailto", blank=True)),
                ("link_phone", models.CharField(max_length=40, null=True, verbose_name="Phone", blank=True)),
                (
                    "link_target",
                    models.CharField(
                        blank=True,
                        max_length=100,
                        verbose_name="target",
                        choices=[
                            ("", "same window"),
                            ("_blank", "new window"),
                            ("_parent", "parent window"),
                            ("_top", "topmost frame"),
                        ],
                    ),
                ),
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("label", models.CharField(default="", max_length=256, verbose_name="label", blank=True)),
                ("type", aldryn_bootstrap3.model_fields.LinkOrButton(default="lnk", max_length=10)),
                (
                    "btn_context",
                    aldryn_bootstrap3.model_fields.Context(
                        default="default",
                        max_length=255,
                        verbose_name="context",
                        blank=True,
                        choices=[
                            ("default", "Default"),
                            ("primary", "Primary"),
                            ("success", "Success"),
                            ("info", "Info"),
                            ("warning", "Warning"),
                            ("danger", "Danger"),
                            ("link", "Link Button"),
                            ("", "Custom"),
                        ],
                    ),
                ),
                (
                    "btn_size",
                    aldryn_bootstrap3.model_fields.Size(default="md", max_length=255, verbose_name="size", blank=True),
                ),
                ("btn_block", models.BooleanField(default=False, verbose_name="block")),
                (
                    "txt_context",
                    aldryn_bootstrap3.model_fields.Context(
                        default="",
                        max_length=255,
                        verbose_name="context",
                        blank=True,
                        choices=[
                            ("", "None"),
                            ("primary", "Primary"),
                            ("success", "Success"),
                            ("info", "Info"),
                            ("warning", "Warning"),
                            ("danger", "Danger"),
                            ("muted", "Muted"),
                        ],
                    ),
                ),
                ("icon_left", aldryn_bootstrap3.model_fields.Icon(default="", max_length=255, blank=True)),
                ("icon_right", aldryn_bootstrap3.model_fields.Icon(default="", max_length=255, blank=True)),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
                ("responsive", aldryn_bootstrap3.model_fields.Responsive(default="", blank=True)),
                ("responsive_print", aldryn_bootstrap3.model_fields.ResponsivePrint(default="", blank=True)),
                (
                    "link_file",
                    filer.fields.file.FilerFileField(
                        on_delete=django.db.models.deletion.SET_NULL,
                        verbose_name="file",
                        blank=True,
                        to="filer.File",
                        null=True,
                    ),
                ),
                (
                    "link_page",
                    cms.models.fields.PageField(
                        on_delete=django.db.models.deletion.SET_NULL,
                        verbose_name="page",
                        blank=True,
                        to="cms.Page",
                        null=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin", models.Model),
        ),
        migrations.CreateModel(
            name="Boostrap3IconPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("icon", aldryn_bootstrap3.model_fields.Icon(default="", max_length=255, blank=True)),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3ImagePlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("alt", aldryn_bootstrap3.model_fields.MiniText(default="", verbose_name="alt", blank=True)),
                ("title", aldryn_bootstrap3.model_fields.MiniText(default="", verbose_name="title", blank=True)),
                (
                    "aspect_ratio",
                    models.CharField(
                        default="",
                        max_length=10,
                        verbose_name="aspect ratio",
                        blank=True,
                        choices=[
                            ("1x1", "1x1"),
                            ("4x3", "4x3"),
                            ("16x9", "16x9"),
                            ("16x10", "16x10"),
                            ("21x9", "21x9"),
                            ("1x1", "1x1"),
                            ("3x4", "3x4"),
                            ("9x16", "9x16"),
                            ("10x16", "10x16"),
                            ("9x21", "9x21"),
                        ],
                    ),
                ),
                (
                    "thumbnail",
                    models.BooleanField(
                        default=False, help_text="add the 'thumbnail' border", verbose_name="thumbnail"
                    ),
                ),
                (
                    "shape",
                    models.CharField(
                        default="",
                        max_length=64,
                        verbose_name="shape",
                        blank=True,
                        choices=[("rounded", "rounded"), ("circle", "circle")],
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
                (
                    "img_responsive",
                    models.BooleanField(
                        default=True,
                        help_text="whether to treat the image as using 100% width of the parent container (sets the img-responsive class).",
                        verbose_name="class: img-responsive",
                    ),
                ),
                (
                    "file",
                    filer.fields.image.FilerImageField(
                        related_name="+",
                        on_delete=django.db.models.deletion.SET_NULL,
                        verbose_name="file",
                        blank=True,
                        to="filer.Image",
                        null=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3LabelPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("label", models.CharField(default="", max_length=256, verbose_name="label", blank=True)),
                (
                    "context",
                    aldryn_bootstrap3.model_fields.Context(
                        default="default",
                        max_length=255,
                        choices=[
                            ("default", "Default"),
                            ("primary", "Primary"),
                            ("success", "Success"),
                            ("info", "Info"),
                            ("warning", "Warning"),
                            ("danger", "Danger"),
                        ],
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3PanelBodyPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3PanelFooterPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3PanelHeadingPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "title",
                    aldryn_bootstrap3.model_fields.MiniText(
                        default="", help_text="Alternatively you can add plugins", verbose_name="title", blank=True
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3PanelPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "context",
                    aldryn_bootstrap3.model_fields.Context(
                        default="default",
                        max_length=255,
                        choices=[
                            ("default", "Default"),
                            ("primary", "Primary"),
                            ("success", "Success"),
                            ("info", "Info"),
                            ("warning", "Warning"),
                            ("danger", "Danger"),
                        ],
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3SpacerPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("size", aldryn_bootstrap3.model_fields.Size(default="md", max_length=255, blank=True)),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Boostrap3WellPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        related_name="+",
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("size", aldryn_bootstrap3.model_fields.Size(default="md", max_length=255, blank=True)),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Bootstrap3AccordionItemPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("title", aldryn_bootstrap3.model_fields.MiniText(default="", verbose_name="title", blank=True)),
                (
                    "context",
                    aldryn_bootstrap3.model_fields.Context(
                        default="default",
                        max_length=255,
                        choices=[
                            ("default", "Default"),
                            ("primary", "Primary"),
                            ("success", "Success"),
                            ("info", "Info"),
                            ("warning", "Warning"),
                            ("danger", "Danger"),
                        ],
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Bootstrap3AccordionPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "index",
                    models.PositiveIntegerField(
                        help_text="index of element that should be opened on page load (leave it empty if none of the items should be opened)",
                        null=True,
                        verbose_name="index",
                        blank=True,
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Bootstrap3CarouselPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "style",
                    models.CharField(
                        default="standard", max_length=50, verbose_name="Style", choices=[("standard", "Standard")]
                    ),
                ),
                (
                    "aspect_ratio",
                    models.CharField(
                        default="",
                        max_length=10,
                        verbose_name="aspect ratio",
                        blank=True,
                        choices=[
                            ("1x1", "1x1"),
                            ("4x3", "4x3"),
                            ("16x9", "16x9"),
                            ("16x10", "16x10"),
                            ("21x9", "21x9"),
                            ("1x1", "1x1"),
                            ("3x4", "3x4"),
                            ("9x16", "9x16"),
                            ("10x16", "10x16"),
                            ("9x21", "9x21"),
                        ],
                    ),
                ),
                (
                    "transition_effect",
                    models.CharField(
                        default="",
                        max_length=50,
                        verbose_name="Transition Effect",
                        blank=True,
                        choices=[("slide", "Slide")],
                    ),
                ),
                (
                    "ride",
                    models.BooleanField(
                        default=True,
                        help_text="Whether to mark the carousel as animating starting at page load.",
                        verbose_name="Ride",
                    ),
                ),
                (
                    "interval",
                    models.IntegerField(
                        default=5000,
                        help_text="The amount of time to delay between automatically cycling an item.",
                        verbose_name="Interval",
                    ),
                ),
                (
                    "wrap",
                    models.BooleanField(
                        default=True, help_text="Whether the carousel should cycle continuously or have hard stops."
                    ),
                ),
                (
                    "pause",
                    models.BooleanField(
                        default=True,
                        help_text="Pauses the cycling of the carousel on mouseenter and resumes the cycling of the carousel on mouseleave.",
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Bootstrap3CarouselSlideFolderPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
                (
                    "folder",
                    filer.fields.folder.FilerFolderField(
                        verbose_name="folder", to="filer.Folder", on_delete=models.CASCADE
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Bootstrap3CarouselSlidePlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("link_url", models.URLField(default="", verbose_name="link", blank=True)),
                (
                    "link_anchor",
                    models.CharField(
                        help_text="Adds this value as an anchor (#my-anchor) to the link.",
                        max_length=128,
                        verbose_name="anchor",
                        blank=True,
                    ),
                ),
                ("link_mailto", models.EmailField(max_length=75, null=True, verbose_name="mailto", blank=True)),
                ("link_phone", models.CharField(max_length=40, null=True, verbose_name="Phone", blank=True)),
                (
                    "link_target",
                    models.CharField(
                        blank=True,
                        max_length=100,
                        verbose_name="target",
                        choices=[
                            ("", "same window"),
                            ("_blank", "new window"),
                            ("_parent", "parent window"),
                            ("_top", "topmost frame"),
                        ],
                    ),
                ),
                ("link_text", models.CharField(max_length=200, verbose_name="link text", blank=True)),
                (
                    "content",
                    djangocms_text_ckeditor.fields.HTMLField(
                        default="",
                        help_text="alternatively add sub plugins as content",
                        verbose_name="Content",
                        blank=True,
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
                (
                    "image",
                    filer.fields.image.FilerImageField(
                        related_name="+",
                        on_delete=django.db.models.deletion.SET_NULL,
                        verbose_name="image",
                        blank=True,
                        to="filer.Image",
                        null=True,
                    ),
                ),
                (
                    "link_file",
                    filer.fields.file.FilerFileField(
                        on_delete=django.db.models.deletion.SET_NULL,
                        verbose_name="file",
                        blank=True,
                        to="filer.File",
                        null=True,
                    ),
                ),
                (
                    "link_page",
                    cms.models.fields.PageField(
                        on_delete=django.db.models.deletion.SET_NULL,
                        verbose_name="page",
                        blank=True,
                        to="cms.Page",
                        null=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin", models.Model),
        ),
        migrations.CreateModel(
            name="Bootstrap3ColumnPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
                ("tag", models.SlugField(default="div")),
                (
                    "xs_col",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="col-xs-", blank=True
                    ),
                ),
                (
                    "xs_offset",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="offset-", blank=True
                    ),
                ),
                (
                    "xs_push",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="push-", blank=True
                    ),
                ),
                (
                    "xs_pull",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="pull-", blank=True
                    ),
                ),
                (
                    "sm_col",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="col-sm-", blank=True
                    ),
                ),
                (
                    "sm_offset",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="offset-", blank=True
                    ),
                ),
                (
                    "sm_push",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="push-", blank=True
                    ),
                ),
                (
                    "sm_pull",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="pull-", blank=True
                    ),
                ),
                (
                    "md_col",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="col-md-", blank=True
                    ),
                ),
                (
                    "md_offset",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="offset-", blank=True
                    ),
                ),
                (
                    "md_push",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="push-", blank=True
                    ),
                ),
                (
                    "md_pull",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="pull-", blank=True
                    ),
                ),
                (
                    "lg_col",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="col-lg-", blank=True
                    ),
                ),
                (
                    "lg_offset",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="offset-", blank=True
                    ),
                ),
                (
                    "lg_push",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="push-", blank=True
                    ),
                ),
                (
                    "lg_pull",
                    aldryn_bootstrap3.model_fields.IntegerField(
                        default=None, null=True, verbose_name="pull-", blank=True
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Bootstrap3ListGroupItemPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("title", aldryn_bootstrap3.model_fields.MiniText(default="", verbose_name="title", blank=True)),
                (
                    "context",
                    aldryn_bootstrap3.model_fields.Context(
                        default="",
                        max_length=255,
                        blank=True,
                        choices=[
                            ("", "None"),
                            ("primary", "Primary"),
                            ("success", "Success"),
                            ("info", "Info"),
                            ("warning", "Warning"),
                            ("danger", "Danger"),
                        ],
                    ),
                ),
                (
                    "state",
                    models.CharField(
                        blank=True,
                        max_length=255,
                        verbose_name="state",
                        choices=[("active", "active"), ("disabled", "disabled")],
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Bootstrap3ListGroupPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
                (
                    "add_list_group_class",
                    models.BooleanField(
                        default=True,
                        help_text="whether to add the list-group and list-group-item classes",
                        verbose_name="class: list-group",
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Bootstrap3RowPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="cms.CMSPlugin",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "classes",
                    aldryn_bootstrap3.model_fields.Classes(
                        default="",
                        help_text='space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>',
                        blank=True,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=("cms.cmsplugin",),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-15 21:40


from django.db import migrations
import djangocms_attributes_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_bootstrap3', '0005_boostrap3imageplugin_use_original_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='boostrap3buttonplugin',
            name='link_attributes',
            field=djangocms_attributes_field.fields.AttributesField(default=dict, verbose_name='Link Attributes'),
        ),
        migrations.AddField(
            model_name='bootstrap3carouselslideplugin',
            name='link_attributes',
            field=djangocms_attributes_field.fields.AttributesField(default=dict, verbose_name='Link Attributes'),
        ),
    ]



# -*- coding: utf-8 -*-


import django.forms.widgets
from django.template.loader import render_to_string

from aldryn_bootstrap3.compat import RadioFieldRenderer
from .conf import settings


class SelectFieldCompatMixin(object):
    """
    This class is only needed for Django < 1.11 compatibility
    """

    @property
    def renderer(self):
        class Renderer(RadioFieldRenderer):
            template_name = self.template_name

            def render(self):
                return render_to_string(self.template_name, {"selects": self})

        return Renderer


class Context(SelectFieldCompatMixin, django.forms.widgets.RadioSelect):
    template_name = "admin/aldryn_bootstrap3/widgets/context.html"


class Size(SelectFieldCompatMixin, django.forms.widgets.RadioSelect):
    template_name = "admin/aldryn_bootstrap3/widgets/size.html"


class LinkOrButton(SelectFieldCompatMixin, django.forms.widgets.RadioSelect):
    template_name = "admin/aldryn_bootstrap3/widgets/link_or_button.html"


class Icon(django.forms.widgets.TextInput):
    def render(self, name, value, attrs=None, **kwargs):
        input_html = super(Icon, self).render(name, value, attrs=attrs, **kwargs)
        if value is None:
            value = ""
        iconset = value.split("-")[0] if value and "-" in value else ""
        iconset_prefexes = [s[1] for s in settings.ALDRYN_BOOTSTRAP3_ICONSETS]
        if len(settings.ALDRYN_BOOTSTRAP3_ICONSETS) and iconset not in iconset_prefexes:
            # invalid iconset! maybe because the iconset was removed from
            # the project. set it to the first in the list.
            iconset = settings.ALDRYN_BOOTSTRAP3_ICONSETS[0][1]
        rendered = render_to_string(
            "admin/aldryn_bootstrap3/widgets/icon.html",
            {
                "input_html": input_html,
                "value": value,
                "name": name,
                "iconset": iconset,
                "is_required": self.is_required,
                "iconsets": settings.ALDRYN_BOOTSTRAP3_ICONSETS,
            },
        )
        return rendered


class MiniTextarea(django.forms.widgets.Textarea):
    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {}
        attrs["cols"] = "120"
        attrs["rows"] = "1"
        super(MiniTextarea, self).__init__(attrs)


class Responsive(django.forms.widgets.Textarea):
    def render(self, name, value, attrs=None):
        widget_html = super(Responsive, self).render(name=name, value=value, attrs=attrs)

        rendered = render_to_string(
            "admin/aldryn_bootstrap3/widgets/responsive.html",
            {
                "widget_html": widget_html,
                "widget": self,
                "value": value,
                "name": name,
                "id": attrs.get("id", None),
                "attrs": attrs,
            },
        )
        return rendered


class ResponsivePrint(django.forms.widgets.Textarea):
    def render(self, name, value, attrs=None):
        widget_html = super(ResponsivePrint, self).render(name=name, value=value, attrs=attrs)

        rendered = render_to_string(
            "admin/aldryn_bootstrap3/widgets/responsive_print.html",
            {
                "widget_html": widget_html,
                "widget": self,
                "value": value,
                "name": name,
                "id": attrs.get("id", None),
                "attrs": attrs,
            },
        )
        return rendered



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_bootstrap3', '0007_auto_20160705_1155'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bootstrap3accordionitemplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='bootstrap3accordionplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselslidefolderplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='bootstrap3carouselslideplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='bootstrap3columnplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='bootstrap3listgroupitemplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='bootstrap3listgroupplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='bootstrap3rowplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

from aldryn_bootstrap3 import __version__

REQUIREMENTS = [
    "django>2.2",
    "django-appconf>=1.0.0",
    "django-cms>=3.5.0",
    "django-filer>=0.9.11",
    "djangocms-text-ckeditor>=3.1.0",
    "djangocms-attributes-field>=0.1.1",
]

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Web Environment",
    "Framework :: Django",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: BSD License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Topic :: Internet :: WWW/HTTP",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    "Topic :: Software Development :: Libraries :: Application Frameworks",
    "Topic :: Software Development :: Libraries :: Python Modules",
]

setup(
    name="aldryn-bootstrap3-resurrected",
    version=__version__,
    author="Divio AG / Updated by Amos Vryhof",
    author_email="amos@vryhofresearch.com",
    url="https://github.com/avryhof/aldryn-bootstrap3",
    license="BSD",
    description=("Adds Bootstrap 3 components as plugins."),
    long_description=open("README.rst").read(),
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIREMENTS,
    classifiers=CLASSIFIERS,
    test_suite="tests.settings.run",
)
